/**
 * Name        : main.c
 * Version     :
 * Description : main definition for FreeRTOS application
 */

/*
 * FreeRTOS includes
 */
#include "system_LPC17xx.h"
#include "LPC17xx.h"
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "onewire.h"

#define configQUEUE_RECEIVE_TASK_PRIORITY  ( tskIDLE_PRIORITY + 2 )
#define configQUEUE_SEND_TASK_PRIORITY     ( tskIDLE_PRIORITY + 3 )

/* The rate at which data is sent to the queue, specified in milliseconds. */
#define mainQUEUE_SEND_FREQUENCY_MS        ( 500 / portTICK_RATE_MS )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH            ( 1 )

/*
 * The tasks as described in the accompanying PDF application note.
 */
static void prvQueueReceiveTask( void *pvParameters );
static void prvQueueSendTask( void *pvParameters );

/*
 * Simple function to set the LED on the LPCXpresso board.
 */
static void prvSetLED(uint8_t on );

/* The queue used by both tasks. */
static xQueueHandle xQueue = NULL;


extern void vuIP_Task( void *pvParameters );


int main(void)
{
  SystemClockUpdate();

  /* Initialise P0_22 for the LED. */
  LPC_PINCON->PINSEL1  &= ( ~( 3 << 12 ) );
  LPC_GPIO0->FIODIR |= ( 1 << 22 );

  /* Create the queue. */
  xQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( unsigned long ) );

  if ( xQueue != NULL )
    {
    /* Start the two tasks as described in the accompanying application
    note. */
    xTaskCreate( prvQueueReceiveTask, ( signed char * ) "Rx", configMINIMAL_STACK_SIZE, NULL, configQUEUE_RECEIVE_TASK_PRIORITY, NULL );
    xTaskCreate( prvQueueSendTask, ( signed char * ) "TX", configMINIMAL_STACK_SIZE, NULL, configQUEUE_SEND_TASK_PRIORITY, NULL );

    /* Start the tasks running. */
    vTaskStartScheduler();
    }

  /* If all is well we will never reach here as the scheduler will now be
  running.  If we do reach here then it is likely that there was insufficient
  heap available for the idle task to be created. */
  for( ;; );
}
/*-----------------------------------------------------------*/


static void prvQueueSendTask( void *pvParameters )
{
  ow_init();
  portTickType xNextWakeTime;
  const unsigned long ulValueToSend = 100UL;

  /* Initialise xNextWakeTime - this only needs to be done once. */
  xNextWakeTime = xTaskGetTickCount();

  for ( ;; )
    {
    RomType aRom;
    // geht nur mit einem Sensor am Bus!
    // das Handling mit dem Sensor passiert im Interrupt
    DS18X22_StartReadRom();
  
    // 50ms warten, da muss der Vorgang zu Ende sein
    // es gibt keine Warteschleifen
    vTaskDelay(50);
  
    // nicht OW_CMD_COMPLETE? Fehler! kein Sensor dran?
    if (DS18X20_GetStatus()!=OW_CMD_COMPLETE)
      continue;
  
    // jetzt die gelesenen Werte mit crc8 überpruefen
    // und in aRom schreiben wenn ok
    if (!DS18X22_GetRom(&aRom))
      continue;
  
    // Messung starten. Dauert mit Parasite Power
    // max. 750ms. 800ms warten
    DS18X22_StartMeasure(&aRom);
    vTaskDelay(800);
  
    // d.t.o
    if (DS18X20_GetStatus()!=OW_CMD_COMPLETE)
      continue;
  
      // das Lesen des Scratchpads anstarten
    DS18X22_StartReadDecicelsius(&aRom);
  
    // 50ms warten, da muss der Vorgang zu Ende sein
    vTaskDelay(50);
  
    // d.t.o
    if (DS18X20_GetStatus()!=OW_CMD_COMPLETE)
      continue;
  
    // Berechnen des Wertes incl. Prüfung der crc8
    uint16_t decicelsius=DS18X22_GetDecicelsius(0x10);

    // 850 liefert der DS18X20 wenn keine gültige Messung 
    // ausgeführt wurde bevor des Scratchpad gelesen wird
    // passiert auch wenn im Parasit-Modus die Pin nicht rechtzeitig
    // hart auf HIGH gezogen wird
    if (   decicelsius == DS18X20_INVALID_DECICELSIUS 
        || decicelsius == 850)
      continue;
  
    xQueueSend( xQueue, &ulValueToSend, 0 );
    }
}
/*-----------------------------------------------------------*/

static void prvQueueReceiveTask( void *pvParameters )
{
  unsigned long ulReceivedValue;
  for ( ;; )
    {
    xQueueReceive( xQueue, &ulReceivedValue, portMAX_DELAY );
    if ( ulReceivedValue == 100UL )
      {
      prvSetLED(1);
      vTaskDelay(200);
      prvSetLED(0);
      }
    }
}
/*-----------------------------------------------------------*/

static void prvSetLED(uint8_t on )
{
  unsigned long ulLEDState;
  ulLEDState = LPC_GPIO0->FIOPIN;
  if (on)
    LPC_GPIO0->FIOSET = ( 1 << 22 ) ;
  else
    LPC_GPIO0->FIOCLR = ( 1 << 22 );
}
/*-----------------------------------------------------------*/


