#include "onewire.h"                                   
#include "crc8.h"


#define  PINSEL     ((volatile uint32_t*)&LPC_PINCON->PINSEL0)
#define  PINMODE     ((volatile uint32_t*)&LPC_PINCON->PINMODE0)
#define  PINMODE_OD ((volatile uint32_t*)&LPC_PINCON->PINMODE_OD0)

#define GPIO_PINSEL(p,b,v)      PINSEL[(p) * 2 + (b) / 16] = (PINSEL[(p) * 2 + (b) / 16] & ~(3 << ((b) * 2 % 32))) | (v << ((b) * 2 % 32))
#define GPIO_PINMODE_IN(p,b,v)  PINMODE[(p) * 2 + (b) / 16] = (PINMODE[(p) * 2 + (b) / 16] & ~(3 << ((b) * 2 % 32))) | (v << ((b) * 2 % 32))
#define GPIO_PINMODE_OUT(p,b,v) PINMODE_OD[(p)] = ((PINMODE_OD[(p)] & ~(1 << b))) | (v << (b))

#define GPIO_IN_PULLUP     0x00
#define GPIO_IN_REPEATER   0x01
#define GPIO_IN_FLOAT      0x02
#define GPIO_IN_PULLDOWN   0x03

#define GPIO_OUT_NORMAL    0x00
#define GPIO_OUT_OPENDRAIN 0x01

#define LPC_GPIO(a)       (LPC_GPIO0 + a*(LPC_GPIO1-LPC_GPIO0))
#define GPIO_DIR_OUT(a,b) (LPC_GPIO(a)->FIODIR |=  (1<<b))
#define GPIO_DIR_IN(a,b)  (LPC_GPIO(a)->FIODIR &= ~(1<<b))
#define GPIO_SET(a,b)     (LPC_GPIO(a)->FIOSET  =  (1<<b)) 
#define GPIO_CLR(a,b)     (LPC_GPIO(a)->FIOCLR  =  (1<<b)) 
#define GPIO_IS_SET(a,b)  ((LPC_GPIO(a)->FIOPIN & (1<<b)) !=0)

#define OW_OPENDRAIN    GPIO_PINMODE_OUT(OW_PORT, OW_PIN, GPIO_OUT_OPENDRAIN)
#define OW_OUTPUTPOWER  GPIO_PINMODE_OUT(OW_PORT, OW_PIN, GPIO_OUT_NORMAL)

#define OW_LOW     GPIO_CLR(OW_PORT, OW_PIN) 
#define OW_HIGH    GPIO_SET(OW_PORT, OW_PIN) 
#define OW_SENSE   GPIO_IS_SET(OW_PORT, OW_PIN) 

#if OW_TIMER == 0
  #define TIMER_IRQn          TIMER0_IRQn
  #define LPC_TIM             LPC_TIM0
  #define TIMER_IRQHandler    TIMER0_IRQHandler
#elif OW_TIMER == 1
  #define TIMER_IRQn          TIMER1_IRQn
  #define LPC_TIM             LPC_TIM1
  #define TIMER_IRQHandler    TIMER1_IRQHandler
#elif OW_TIMER == 2
  #define TIMER_IRQn          TIMER2_IRQn
  #define LPC_TIM             LPC_TIM2
  #define TIMER_IRQHandler    TIMER2_IRQHandler
#elif OW_TIMER == 3
  #define TIMER_IRQn          TIMER3_IRQn
  #define LPC_TIM             LPC_TIM3
  #define TIMER_IRQHandler    TIMER3_IRQHandler
#endif

// Match Control register bit positions
#define MR0I 0
#define MR1I 3
#define MR2I 6

#define CMD_ROM   0x033    // read rom
#define CMD_CONV  0x044    // convert temperature
#define CMD_MATCH 0x055    // match rom
#define CMD_PWR   0x0b4    // read power
#define CMD_READ  0x0be    // read scratch pad
#define CMD_SKIP  0x0cc    // skip rom

static volatile uint8_t owbuf[10];
static volatile uint8_t  nwrite;
static volatile uint8_t  nread;

static volatile TOWState ow_state;
static volatile uint8_t  presence;
static volatile uint8_t  data_index;
static volatile uint8_t  cur_byte;
static volatile uint8_t  bit_pos;
static volatile uint8_t  ow_busy;

void ow_init()
{
#if OW_TIMER == 0
  LPC_SC->PCONP |= (1<<1);
  LPC_SC->PCLKSEL0 &= ~(0x00<<2); // CCLK Vorteiler 4 an Timer 0
#elif OW_TIMER == 1
  LPC_SC->PCONP |= (1<<2);
  LPC_SC->PCLKSEL0 &= ~(0x00<<4); // CCLK Vorteiler 4 an Timer 1
#elif OW_TIMER == 2
  LPC_SC->PCONP |= (1<<22);
  LPC_SC->PCLKSEL0 &= ~(0x00<<12); // CCLK Vorteiler 4 an Timer 2
#elif OW_TIMER == 3
  LPC_SC->PCONP |= (1<<23);
  LPC_SC->PCLKSEL0 &= ~(0x00<<14); // CCLK Vorteiler 4 an Timer 3
#endif



  // Pin ist normaler Gpio
  GPIO_PINSEL(OW_PORT, OW_PIN, 0);

  // Pullup und Pulldown abschalten
  GPIO_PINMODE_IN(OW_PORT, OW_PIN, GPIO_IN_FLOAT);

  // der Pin bleibt immer im Zustand Ausgang
  // und Opendrain Modus. Zum Lesen des Pins muss
  // in dieser Konfiguration nicht auf
  // Eingang umgeschaltet werden
  OW_HIGH;
  GPIO_DIR_OUT(OW_PORT, OW_PIN);
  GPIO_PINMODE_OUT(OW_PORT, OW_PIN, GPIO_OUT_OPENDRAIN);

  LPC_TIM->PR =  (SystemFrequency/1000000)/4;
  LPC_TIM->TCR = 0x02; // timer stopped and reset

  // Enable the TIMER0 Interrupt
  // and change Priority
  NVIC_SetPriority(TIMER_IRQn, TIMER_IRQ_PRIORIOTY);
  NVIC_EnableIRQ(TIMER_IRQn);
}

void TIMER_IRQHandler(void)
{
  switch (ow_state) 
    {
    case OW_START:
      {
      if (LPC_TIM->IR==0x01)
        {
        LPC_TIM->IR = 0x01;                 // clear interrupt
        ow_busy = 1;         
        presence = 0;
        LPC_TIM->TCR = 0x02;                // timer stopped and reset
        LPC_TIM->MCR = 0;                   // clear match control flags
        ow_state = OW_RESET;
        LPC_TIM->MR0 = 550;                 // was 480    reset drive duration
        LPC_TIM->MR1 = 550 + 80;            // was 480+80 presence pulse detection
        LPC_TIM->MR2 = 2180;                // was 960    reset finished
        LPC_TIM->MCR = (1 << MR0I) | (1 << MR1I) | (1 << MR2I);
        // am Ende von CMD_CONV wird der Pin bei parasitärem Power 
        // als Output configuriert. Das muss Am Anfang des folgenden Befehls 
        // rückgängig gemacht werden        
        OW_OPENDRAIN;
        OW_LOW;              // drive low
        LPC_TIM->TCR = 0x01; // start timer
        return;
        } 
      ow_state=OW_ERROR;
      break;
      }        

    case OW_RESET:
      {
      switch (LPC_TIM->IR) 
        {
        case 0x01:
          LPC_TIM->IR = 0x01;                 // clear interrupt
          OW_HIGH;                            // set pin high
          return;
        case 0x02:
          LPC_TIM->IR = 0x02;                 // clear interrupt
          if (OW_SENSE == 0)
            presence = 1;
          return;
        case 0x04:
          LPC_TIM->IR =  0x04;                // clear interrupt
          LPC_TIM->TCR = 0x02;                // timer stopped and reset
          LPC_TIM->MCR = 0;                   // clear match control flags
          if (presence==0)
            {
            ow_state = OW_NO_DEVICE;          // no device found
            break;
            }
           
          if (nwrite)
            {
            data_index = 0;
            bit_pos = 0;
            cur_byte = owbuf[data_index];
            ow_state = OW_WRITE;
            LPC_TIM->MR0 = 3;                 // was 2 end of write pulse
            LPC_TIM->MR1 = 60;                // end of write cycle
            LPC_TIM->MR2 = 164;               // was 62 ready to start next write cycle
            LPC_TIM->MCR = (1 << MR0I) | (1 << MR1I) | (1 << MR2I);
            OW_LOW;                           // drive low
            LPC_TIM->TCR = 0x01;              // start timer
            }           
          return;
        
        default:
          ow_state=OW_ERROR;
          break;
        }
      break; // end of reset case
      }
      
    case OW_WRITE:
      {
      switch (LPC_TIM->IR) 
        {
        case 0x01:                            // end of write pulse
          LPC_TIM->IR = 0x01;                 // clear interrupt
          if ((cur_byte >> bit_pos) & 0x01) 
            OW_HIGH;                          // set pin high else low
          return;
        case 0x02:                            // end of write cycle
          LPC_TIM->IR = 0x02;                 // clear interrupt
          OW_HIGH;                            // set pin high
          #ifdef POWER_PARASITE
          if (cur_byte==CMD_CONV && bit_pos==7)
            OW_OUTPUTPOWER;
          #endif  
          return;
        case 0x04:                            // start next write cycle
          LPC_TIM->IR = 0x04;                 // clear interrupt
          LPC_TIM->TCR = 0x02;                // timer stopped and reset
          bit_pos++;
          if (bit_pos == 8) 
            {
            // end of byte
            bit_pos = 0;
            data_index++;
            if (data_index == nwrite) 
              {
              // Schreiben zu Ende es soll sofort gelesen werden              
              if (nread)
                {
                data_index = 0;
                bit_pos =  0;
                cur_byte = 0;
                ow_state = OW_READ;
                LPC_TIM->MR0 = 2;              // was 2  end of read pulse
                LPC_TIM->MR1 = 16;             // time to sample line
                LPC_TIM->MR2 = 166;             // ready to start next read cycle
                LPC_TIM->MCR = (1 << MR0I) | (1 << MR1I) | (1 << MR2I);
                OW_LOW;                        // drive low
                LPC_TIM->TCR = 0x01;           // start timer
                return;
                } 
              else
                {              
                ow_state = OW_CMD_COMPLETE;
                LPC_TIM->TCR = 0x02;           // timer stopped and reset
                LPC_TIM->MCR = 0;              // clear match control flags
                break;
                }
              } 
            else 
              {
              // next byte
              cur_byte = owbuf[data_index];
              }
            }
          LPC_TIM->TCR = 0x02;                // reset timer
          OW_LOW;                             // drive low
          LPC_TIM->TCR = 0x01;                // restart timer
          return;
       
        default:                              // start next write cycle
          ow_state = OW_ERROR;
          break;
        }
      break; // end of write case
      }  
    
    case OW_READ:
      {
      switch (LPC_TIM->IR) 
        {
        case 0x01:                            // end of read pulse
          LPC_TIM->IR = 0x01;                 // clear interrupt
          OW_HIGH;                            // set pin high
          return;
        
        case 0x02:                            // time to sample line
          LPC_TIM->IR = 0x02;                 // clear interrupt
          if (OW_SENSE) 
            cur_byte |= (1 << bit_pos);
          return;
        
        case 0x04:                           // start next read cycle
          LPC_TIM->IR = 0x04;                // clear interrupt
          bit_pos++;
          if (bit_pos == 8) 
            {
            // end of byte
            owbuf[data_index] = cur_byte;
            cur_byte = 0;
            bit_pos = 0;
            data_index++;
            if (data_index == nread) 
              {
              // end of data
              ow_state = OW_CMD_COMPLETE;
              break;
              }
            }
          LPC_TIM->TCR = 0x02;                // reset timer
          OW_LOW;                             // drive low
          LPC_TIM->TCR = 0x01;                // restart timer
          return;

        default:                              // start next write cycle
          ow_state = OW_ERROR;
          break;
        }
      break; // end of read case
      }
    
    default:
      {
      ow_state = OW_ERROR;
      break;
      }
    }

  // wird nur erreicht bei fehlern oder 
  // am ende  
  LPC_TIM->IR =  LPC_TIM->IR; // clear interrupt
  LPC_TIM->TCR = 0x02; // reset timer
  LPC_TIM->MCR = 0;    // clear match control flags
  ow_busy=0;
  return;
}


#define DS18X20_SP_SIZE 9  

void ow_start()
{
  presence = 0;
  ow_busy=1;
  ow_state = OW_START;
  LPC_TIM->TCR = 0x02; // reset timer
  LPC_TIM->MR0 = 2;
  LPC_TIM->MCR = (1 << MR0I);
  LPC_TIM->TCR = 0x01; // start timer
}


void DS18X22_StartReadScratchPad(RomType *rom_ptr)
{
  uint8_t i;

  if (rom_ptr == NULL) 
    {
    owbuf[0] = CMD_SKIP;
    owbuf[1] = CMD_READ;
    nwrite=2;
    } 
  else 
    {
    owbuf[0] = CMD_MATCH;
    for (i = 0; i < 8; i++) 
      owbuf[i + 1] = rom_ptr->b[i];
    owbuf[9] = CMD_READ;
    nwrite=10;
    }
  nread=DS18X20_SP_SIZE;
  ow_start();
}

void DS18X22_StartMeasure(RomType *rom_ptr)
{
  uint8_t i;

  if (rom_ptr == NULL) 
    {
    owbuf[0] = CMD_SKIP;
    owbuf[1] = CMD_CONV;
    nwrite=2;
    } 
  else 
    {
    owbuf[0] = CMD_MATCH;
    for (i = 0; i < 8; i++) 
      owbuf[i + 1] = rom_ptr->b[i];
    owbuf[9] = CMD_CONV;
    nwrite=10;
    }
  nread=0;
  ow_start();
}

void DS18X22_StartReadRom()
{
  owbuf[0] = CMD_ROM;
  nwrite=1;
  nread=8;
  ow_start();
}

uint8_t DS18X22_GetRom(RomType *rom_ptr)
{
  if (ow_state != OW_CMD_COMPLETE)
	return 0;

  int crc=crc8( (uint8_t*)owbuf, 8);
  if (crc!=0)
    return 0;
  int i;
  for (i = 0; i < 8; i++)
    rom_ptr->b[i]=owbuf[i];
  return 1;
}


// DS18X20 specific values (see datasheet) 
#define DS18S20_FAMILY_CODE       0x10
#define DS18B20_FAMILY_CODE       0x28
#define DS1822_FAMILY_CODE        0x22

#define DS18B20_CONF_REG          4
#define DS18B20_9_BIT             0
#define DS18B20_10_BIT            (1<<5)
#define DS18B20_11_BIT            (1<<6)
#define DS18B20_12_BIT            ((1<<6)|(1<<5))
#define DS18B20_RES_MASK          ((1<<6)|(1<<5))

// undefined bits in LSB if 18B20 != 12bit
#define DS18B20_9_BIT_UNDF        ((1<<0)|(1<<1)|(1<<2))
#define DS18B20_10_BIT_UNDF       ((1<<0)|(1<<1))
#define DS18B20_11_BIT_UNDF       ((1<<0))
#define DS18B20_12_BIT_UNDF       0
#define DS18X20_INVALID_DECICELSIUS  2000

static int16_t DS18X20_RawToDecicelsius( uint8_t familycode, uint8_t sp[] )
{
  uint16_t measure;
  uint8_t  negative;
  int16_t  decicelsius;
  uint16_t fract;

  measure = sp[0] | (sp[1] << 8);
 
  if( familycode == DS18S20_FAMILY_CODE ) 
    {   // 9 -> 12 bit if 18S20
    // Extended measurements for DS18S20 contributed by Carsten Foss 
    measure &= (uint16_t)0xfffe;   // Discard LSB, needed for later extended precicion calc
    measure <<= 3;                 // Convert to 12-bit, now degrees are in 1/16 degrees units
    measure += (16 - sp[6]) - 4;   // Add the compensation and remember to subtract 0.25 degree (4/16)
    }

  // check for negative
  if ( measure & 0x8000 )  
    {
    negative = 1;       // mark negative
    measure ^= 0xffff;  // convert to positive => (twos complement)++
    measure++;
    }
  else 
    {
    negative = 0;
    }

  // clear undefined bits for DS18B20 != 12bit resolution
  if ( familycode == DS18B20_FAMILY_CODE || familycode == DS1822_FAMILY_CODE ) 
    {
    switch( sp[DS18B20_CONF_REG] & DS18B20_RES_MASK ) 
      {
      case DS18B20_9_BIT:
        measure &= ~(DS18B20_9_BIT_UNDF);
        break;
      case DS18B20_10_BIT:
        measure &= ~(DS18B20_10_BIT_UNDF);
        break;
      case DS18B20_11_BIT:
        measure &= ~(DS18B20_11_BIT_UNDF);
        break;
      default:
        // 12 bit - all bits valid
        break;
      }
    }

  decicelsius = (measure >> 4);
  decicelsius *= 10;

  // decicelsius += ((measure & 0x000F) * 640 + 512) / 1024;
  // 625/1000 = 640/1024
  fract = ( measure & 0x000F ) * 640;
  if ( !negative ) 
    {
    fract += 512;
    }
  fract /= 1024;
  decicelsius += fract;

  if ( negative ) 
    decicelsius = -decicelsius;
  
  if ( /* decicelsius == 850 || */ decicelsius < -550 || decicelsius > 1250 ) 
    return DS18X20_INVALID_DECICELSIUS;
  return decicelsius;
}

int16_t DS18X22_ReadDecicelsius(uint8_t familycode, RomType *rom_ptr)
{
  DS18X22_StartReadScratchPad(0);
  while (ow_busy)
    ;

  int crc=crc8( (uint8_t*)owbuf, DS18X20_SP_SIZE); //DS18X20_SP_SIZE );
  if (crc!=0)  
    return DS18X20_INVALID_DECICELSIUS;

  return DS18X20_RawToDecicelsius(familycode,(uint8_t*)owbuf);
}

uint8_t DS18X20_GetStatus()
{
  return ow_state;
}

void DS18X22_StartReadDecicelsius(RomType *rom_ptr)
{
  DS18X22_StartReadScratchPad(rom_ptr);
}

uint16_t DS18X22_GetDecicelsius(uint8_t familycode)
{
  int crc=crc8( (uint8_t*)owbuf, DS18X20_SP_SIZE); //DS18X20_SP_SIZE );
  if (crc!=0)
    return DS18X20_INVALID_DECICELSIUS;
  return DS18X20_RawToDecicelsius(familycode,(uint8_t*)owbuf);
}


uint16_t  DS18X22_ReadDecicelsiusBlocking(uint8_t familycode, RomType *rom_ptr)
{
  DS18X22_StartReadScratchPad(rom_ptr);
  while (ow_busy)
    ;

  int crc=crc8( (uint8_t*)owbuf, DS18X20_SP_SIZE); //DS18X20_SP_SIZE );
  if (crc!=0)
    return DS18X20_INVALID_DECICELSIUS;
  return DS18X20_RawToDecicelsius(familycode,(uint8_t*)owbuf);
}




uint8_t sp[10];
void main_test()
{
  int crc;
  ow_init();
  int decicelius;
  while (1)
    {
    
    
    DS18X22_StartReadRom();

    while (ow_busy)
      ;
    crc=crc8( (uint8_t*)owbuf, 8); //DS18X20_SP_SIZE );


    DS18X22_StartMeasure(0);
    while (ow_busy)
      ;
    

    decicelius=DS18X22_ReadDecicelsius(DS18S20_FAMILY_CODE, 0);
    }
  return;
}
  
